@@ -17,15 +17,13 @@ contributions or suggestions of new formats are very welcome.
 
 TODOs:
  * Identify HERWIG decay matrix element to use in ISAWIG
- * Order decay products as required by HERWIG in ISAWIG
- * What to do about unlisted PIDs?
  * Split writeSLHA into writeSLHA{Blocks,Decays}
  * Handle SLHA2
  * Handle RPV SUSY in ISAWIG
 """
 
 __author__ = "Andy Buckley <andy.buckley@cern.ch"
-__version__ = "0.4.1"
+__version__ = "0.4.3"
 
 
 def _autotype(var):
@@ -631,23 +629,62 @@ def writeISAWIG(blocks, decays, ignorenobr=False):
             ## Skip decay if it has no branching ratio
             if ignorenobr and d.br == 0:
                 continue
-            ## TODO: Identify decay matrix element to use
+
+            ## Identify decay matrix element to use
             ## From std HW docs, or from this pair:
             ## Two new matrix element codes have been added for these new decays:
             ##    NME =	200 	3 body top quark via charged Higgs
             ##    	300 	3 body R-parity violating gaugino and gluino decays
-            ## How to determine which to use?
-            #decayout += str(i_d + 1) + " "
-            decayout += "%d %e 0 " % (hwid, d.br)
+            nme = 0
+            # TODO: Get correct condition for using ME 100... this guessed from some ISAWIG output
+            if abs(pid) in (6, 12):
+                nme = 100
+            ## Extra SUSY MEs
+            if len(d.ids) == 3:
+                # TODO: How to determine the conditions for using 200 and 300 MEs? Enumeration of affected decays?
+                pass
+            decayout += "%d %e %d " % (hwid, d.br, nme)
+
+            def is_quark(pid):
+                return (abs(pid) in range(1, 7))
+
+            def is_lepton(pid):
+                return (abs(pid) in range(11, 17))
+
+            def is_squark(pid):
+                if abs(pid) in range(1000001, 1000007):
+                    return True
+                if abs(pid) in range(2000001, 2000007):
+                    return True
+                return False
+
+            def is_slepton(pid):
+                if abs(pid) in range(1000011, 1000017):
+                    return True
+                if abs(pid) in range(2000011, 2000016, 2):
+                    return True
+                return False
+
+            def is_gaugino(pid):
+                if abs(pid) in range(1000022, 1000026):
+                    return True
+                if abs(pid) in (1000035, 1000037):
+                    return True
+                return False
+
+            def is_susy(pid):
+                return (is_squark(pid) or is_slepton(pid) or is_gaugino(pid) or pid == 1000021)
+
+            absids = map(abs, d.ids)
+
             ## Order decay products as required by HERWIG
             ## Top
-            absids = map(abs, d.ids)
             if abs(pid) == 6:
                 def cmp_bottomlast(a, b):
                     """Comparison function which always puts b/bbar last"""
                     if abs(a) == 5:
                         return True
-                    elif abs(b) == 5:
+                    if abs(b) == 5:
                         return False
                     return cmp(a, b)
                 if len(absids) == 2:
@@ -669,37 +706,76 @@ def writeISAWIG(blocks, decays, ignorenobr=False):
                             """Comparison function which always puts gluon first"""
                             if a == 21:
                                 return False
-                            elif b == 21:
+                            if b == 21:
                                 return True
                             return cmp(a, b)
                         d.ids = sorted(d.ids, key=cmp_gluonfirst)
                 elif len(absids) == 3:
                     ## 3-body modes, R-parity conserved: colour neutral; q or qbar
-                    pass
-
-            ## TODO: More sorting!
+                    def cmp_quarkslast(a, b):
+                        """Comparison function which always puts quarks last"""
+                        if is_quark(a):
+                            return True
+                        if is_quark(b):
+                            return False
+                        return cmp(a, b)
+                    d.ids = sorted(d.ids, key=cmp_quarkslast)
             ## Squark/Slepton
-            ##   2 body modes: Gaugino/Gluino with Quark/Lepton     Gaugino      quark
-            ##                                                      Gluino       lepton
-            ##   3 body modes: Weak                                 sparticle     particles from W decay
-            ## Squark
-            ##   2 body modes: Lepton Number Violated               quark     lepton
-            ##                 Baryon number violated               quark     quark
-            ## Slepton
-            ##   2 body modes: Lepton Number Violated               q or qbar
+            elif is_squark(pid) or is_slepton(pid):
+                def cmp_susy_quark_lepton(a, b):
+                    if is_susy(a):
+                        return False
+                    if is_susy(b):
+                        return True
+                    if is_quark(a):
+                        return False
+                    if is_quark(b):
+                        return True
+                    return cmp(a, b)
+                ##   2 body modes: Gaugino/Gluino with Quark/Lepton     Gaugino      quark
+                ##                                                      Gluino       lepton
+                ##   3 body modes: Weak                                 sparticle    particles from W decay
+                ## Squark
+                ##   2 body modes: Lepton Number Violated               quark     lepton
+                ##                 Baryon number violated               quark     quark
+                ## Slepton
+                ##   2 body modes: Lepton Number Violated               q or qbar
+                d.ids = sorted(d.ids, key=cmp_bottomlast)
             ## Higgs
-            ##   2 body modes: (s)quark-(s)qbar                     (s)q or (s)qbar
-            ##                 (s)lepton-(s)lepton                  (s)l or (s)lbar
-            ##   3 body modes: Colour neutral                       q or qbar
-            ##                                                      l or lbar
-            ## Gaugino
-            ##   2 body modes: Squark-quark                         q or sq
-            ##                 Slepton-lepton                       l or sl
-            ##
-            ##   3 body modes: R-parity conserved   colour neutral  q or qbar
-            ##                                                      l or lbar
+            elif pid in (25, 26):
+                # TODO: Includes SUSY Higgses?
+                ## Higgs
+                ##   2 body modes: (s)quark-(s)qbar                     (s)q or (s)qbar
+                ##                 (s)lepton-(s)lepton                  (s)l or (s)lbar
+                ##   3 body modes:                                      colour neutral       q or qbar
+                if len(absids) == 3:
+                    def cmp_quarkslast(a, b):
+                        """Comparison function which always puts quarks last"""
+                        if is_quark(a):
+                            return True
+                        if is_quark(b):
+                            return False
+                        return cmp(a, b)
+                    d.ids = sorted(d.ids, key=cmp_quarkslast)
+            elif is_gaugino(pid):
+                # TODO: Is there actually anything to do here?
+                ## Gaugino
+                ##   2 body modes: Squark-quark                         q or sq
+                ##                 Slepton-lepton                       l or sl
+                ##
+                ##   3 body modes: R-parity conserved                   colour neutral       q or qbar
+                ##                                                                           l or lbar
+                if len(absids) == 3:
+                    def cmp_quarkslast(a, b):
+                        """Comparison function which always puts quarks last"""
+                        if is_quark(a):
+                            return True
+                        if is_quark(b):
+                            return False
+                        return cmp(a, b)
+                    d.ids = sorted(d.ids, key=cmp_quarkslast)
 
-            ## TODO: Gaugino/Gluino
+            # TODO: Gaugino/Gluino
             ##   3 body modes:  R-parity violating:   Particles in the same order as the R-parity violating superpotential
 
             ## Pad out IDs list with zeros
